/**
 * Copyright (c) 2003, Brian McCallister
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * Neither the name of the copyright holder nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 *
 *THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.skife.intercept.interceptors.security;

import java.util.ArrayList;
import java.util.Iterator;
import java.lang.reflect.Method;

/**
 * <code>SecurityBroker</code> is used to add thread constrained actor information to
 */
public class SecurityBroker
{
    private ThreadLocal actor;
    private ArrayList rules;

    public SecurityBroker()
    {
        this.rules = new ArrayList();
        this.actor = new ThreadLocal()
        {
            protected Object initialValue()
            {
                return new Actor() {};
            }
        };
    }

    /**
     * Set the current thread's actor
     * @param actor
     */
    public void setActor(Actor actor)
    {
        this.actor.set(actor);
    }

    public Actor getActor()
    {
        return (Actor) actor.get();
    }

    public void appendRule(Rule rule)
    {
        this.rules.add(rule);
    }

    public void prependRule(Rule rule)
    {
        this.rules.add(0, rule);
    }

    public boolean test(Method m, Object object, Object[] args)
    {
        InvocationContext ctx = new InvocationContext(this.getActor(), m, object, args);
        for (Iterator i = this.rules.iterator(); i.hasNext();)
        {
            Rule rule = (Rule) i.next();
            if (! rule.allow(ctx)) return false;
        }
        return true;
    }
}
