/**
 * Copyright (c) 2003, Brian McCallister
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * Neither the name of the copyright holder nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 *
 *THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.skife.intercept.interceptors.tx;

import org.odmg.Implementation;

/**
 * Provides a wrapper around ODMG transactions for the TransactionInterceptor
 */
public class ODMGTransactionFactory implements TransactionFactory
{
    private Implementation impl;

    /**
     * Create a new ODMGTransactionFactory
     */
    public ODMGTransactionFactory(Implementation impl)
    {
        this.impl = impl;
    }

    /**
     * For Type 2 IoC
     * <p>
     * If this constructor is used the <code>setImplementation(Implementation)</code>
     * method <b>must</b> be called.
     */
    public ODMGTransactionFactory()
    {
        // Nothing
    }

    /**
     * Specify the ODMG <code>Implementation</code> to use
     * @param impl
     */
    public void setImplementation(Implementation impl)
    {
        this.impl = impl;
    }

    /**
     * Obtain the current transaction
     */
    public Transaction build()
    {
        return new ODMGTransaction();
    }

    /**
     * ODMG Transaction wrapper
     */
    private class ODMGTransaction implements Transaction
    {
        public void begin()
        {
            impl.currentTransaction().begin();
        }

        public void commit()
        {
            impl.currentTransaction().commit();
        }

        public void rollback()
        {
            impl.currentTransaction().abort();
        }

        public void checkpoint()
        {
            impl.currentTransaction().checkpoint();
        }

        public boolean isOpen()
        {
            return impl.currentTransaction().isOpen();
        }
    }
}
