package org.skife.intercept.mock;

import com.mockobjects.dynamic.ConstraintMatcher;
import org.skife.intercept.InterceptionBroker;
import org.skife.intercept.Interceptor;
import org.skife.intercept.InterceptorStack;
import org.skife.intercept.interceptors.BlockingInterceptor;
import org.skife.intercept.signatures.EverythingSignature;
import org.skife.intercept.signatures.NamedSignature;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;

public class Mocker
{
    private Object base;
    private InterceptionBroker broker;
    boolean pass = true;

    public Mocker(Object base)
    {
        this.base = base;
        this.broker = new InterceptionBroker();
    }

    public Mocker(Class iface)
    {
        this(new Class[]{iface});
    }

    public Mocker(Class[] ifaces)
    {
        base = Proxy.newProxyInstance(this.getClass().getClassLoader(), ifaces, new InvocationHandler()
        {
            public Object invoke(Object proxy, Method method, Object[] args)
                    throws Throwable
            {
                return null;
            }

        });
        this.broker = new InterceptionBroker();
    }

    public void expect(String name, final ConstraintMatcher matcher)
    {
        this.broker.addInterceptor(new Interceptor()
        {
            public Object intercept(Object base, Method method, Object[] args, InterceptorStack stack) throws Throwable
            {
                if (!matcher.matches(args)) pass = false;
                return null;
            }

        }, new NamedSignature(name));
    }

    public void expect(String name)
    {
        this.broker.addInterceptor(new Interceptor()
        {
            public Object intercept(Object base, Method method, Object[] args, InterceptorStack stack) throws Throwable
            {
                if (args.length > 0) pass = false;
                return null;
            }

        }, new NamedSignature(name));
    }

    public void expectAndReturn(String name, final Object ret)
    {
        this.broker.addInterceptor(new Interceptor()
        {
            public Object intercept(Object base, Method method, Object[] args, InterceptorStack stack) throws Throwable
            {
                if (args.length > 0) pass = false;
                return ret;
            }

        }, new NamedSignature(name));
    }

    public void expectAndReturn(String name, final ConstraintMatcher matcher, final Object ret)
    {
        this.broker.addInterceptor(new Interceptor()
        {
            public Object intercept(Object base, Method method, Object[] args, InterceptorStack stack) throws Throwable
            {
                if (!matcher.matches(args)) pass = false;
                return ret;
            }

        }, new NamedSignature(name));
    }

    public void expectAndReturn(String name, final ConstraintMatcher matcher, final int ret)
    {
        this.broker.addInterceptor(new Interceptor()
        {
            public Object intercept(Object base, Method method, Object[] args, InterceptorStack stack) throws Throwable
            {
                if (!matcher.matches(args)) pass = false;
                return new Integer(ret);
            }

        }, new NamedSignature(name));
    }

    public void expectAndReturn(String name, final ConstraintMatcher matcher, final boolean ret)
    {
        this.broker.addInterceptor(new Interceptor()
        {
            public Object intercept(Object base, Method method, Object[] args, InterceptorStack stack) throws Throwable
            {
                if (!matcher.matches(args)) pass = false;
                return new Boolean(ret);
            }

        }, new NamedSignature(name));
    }

    public Object getProxy()
    {
        this.broker.addInterceptor(new BlockingInterceptor("Unspecified call executed!"),
                                   new EverythingSignature());
        return this.broker.wrap(base);
    }

    public void verify()
    {
        if (!pass) throw new Error();
    }
}
