/**
 * Copyright (c) 2003, Brian McCallister
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * Neither the name of the copyright holder nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 *
 *THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.skife.mixin;

import net.sf.cglib.proxy.Mixin;

import java.util.ArrayList;

/**
 * Provides mixin facility for Java
 */
public class Mixer
{
    private ArrayList ifaces = new ArrayList();
    private ArrayList impls = new ArrayList();

    /**
     * Add all of the registered mixins to an object, returning the mixed object
     *
     * @param base object to add mixins too, is not actually modified
     * @return object implementing all of the currently set mixins
     */
    public Object mix(final Object base)
    {
        Class[] mixins = new Class[ifaces.size()];
        System.arraycopy(ifaces.toArray(), 0, mixins, 0, ifaces.size());
        return Mixin.create(mixins, impls.toArray());
    }

    /**
     * Register a mixin with this Mixer so that all objects mixed
     * after this mixin is added will also have this facility
     *
     * @param iface Interface to mix in
     * @param impl Implementation of iface to delegate to
     */
    public void addMixin(Class iface, Object impl)
    {
        ifaces.add(iface);
        impls.add(impl);
    }
}
