
import org.skife.intercept.benchmark.InterceptMe;
import org.skife.intercept.benchmark.InterceptionProvider;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.PrintWriter;
import java.util.Iterator;
import java.util.Properties;

public class Benchmark
{
    private static final long COUNT = 1000000;
    private static StringBuffer results;

    private static void run(InterceptMe intercept)
    {
        long start = 0, stop = 0;
        start = System.currentTimeMillis();
        for (long i = 0; i < COUNT; ++i)
        {
            intercept.call();
        }
        stop = System.currentTimeMillis();
        results.append("<time>")
                .append(stop - start)
                .append("</time>");

    }

    public static void main(String[] args) throws Exception
    {
        results = new StringBuffer();
        Properties props = new Properties();
        props.load(new FileInputStream(args[0]));

        results.append("<report>");
        for (Iterator i = props.keySet().iterator(); i.hasNext();)
        {
            String name = (String) i.next();
            InterceptionProvider provider =
                    (InterceptionProvider) Class.forName(props.getProperty(name)).newInstance();
            results.append("<library name=\"").append(name).append("\">");
            for (int p = 0; p <= 16; p = (p == 0 ? 1 : (p * 2)))
            {
                InterceptMe fast = provider.build(p, new FastInterceptMe());
                InterceptMe slow = provider.build(p, new SlowInterceptMe());
                results.append("<run interceptors=\"").append(p).append("\">");
                results.append("<fast>");
                run(fast);
                results.append("</fast>");
                results.append("<slow>");
                run(slow);
                results.append("</slow>");
                results.append("</run>");
            }
            results.append("</library>");

        }
        results.append("</report>");
        PrintWriter out = null;
        System.err.println(args.length);
        if (args.length == 1)
        {
            out = new PrintWriter(System.out);
        }
        else
        {
            out = new PrintWriter(new FileOutputStream(args[1]));
        }

        out.println(results.toString());
        out.flush();
    }

    public static class FastInterceptMe implements InterceptMe
    {
        private long count = 0;

        public void call()
        {
            ++count;
        }
    }

    public static class SlowInterceptMe implements InterceptMe
    {
        public void call()
        {
            String foo = this.toString();
        }
    }

}
