package org.skife.intercept.interceptors.security;

import com.mockobjects.dynamic.Mock;
import com.mockobjects.dynamic.AnyConstraintMatcher;
import junit.framework.TestCase;
import org.skife.intercept.InterceptionBroker;
import org.skife.intercept.signatures.NamedSignature;

import java.util.Map;

public class TestSecurityInterceptor extends TestCase
{
    private Map wrap = null;
    private Mock mock = null;
    private SecurityInterceptor intercept = null;
    private InterceptionBroker broker = null;
    private SecurityBroker sec = null;
    private Actor actor = null;

    public void setUp()
    {
        actor = new Actor() {};
        sec = new SecurityBroker();
        sec.setActor(actor);
        intercept = new SecurityInterceptor(sec);
        broker = new InterceptionBroker();
        broker.addInterceptor(intercept, new NamedSignature("put"));
        mock = new Mock(Map.class);
        wrap = (Map) broker.wrap(mock.proxy());
    }

    public void testBlockAccess() throws Exception
    {
        sec.appendRule(Rules.negate(Rules.actorSignatureMatch(new ActorFilter()
        {
            public boolean match(Actor actor)
            {
                return true;
            }

        }, new NamedSignature("put"))));

        try
        {
            wrap.put("1", "1");
            fail("Should have thrown an AccessError");
        }
        catch (AccessError e)
        {
            assertTrue("AccessError was correctly thrown", true);
        }
        mock.verify(); // No calls were made against the mock
    }

    public void testAllowAccess() throws Exception
    {
        mock.expect("put", new AnyConstraintMatcher());
        sec.appendRule(Rules.actorSignatureMatch(new ActorFilter()
        {
            public boolean match(Actor actor)
            {
                return true;
            }

        }, new NamedSignature("put")));

        try
        {
            wrap.put("1", "1");
            assertTrue("AccessError was correctly thrown", true);
        }
        catch (AccessError e)
        {
            fail("Should not have thrown an AccessError");
        }
        mock.verify(); // put was called against the mock
    }
}
