package org.skife.intercept.interceptors.tx;

import com.mockobjects.dynamic.Mock;
import junit.framework.TestCase;
import org.skife.intercept.InterceptionBroker;
import org.skife.intercept.signatures.NamedSignature;

import java.util.HashMap;

public class TestTransactions extends TestCase
{
    public void testBeginCommit() throws Exception
    {
        TransactionInterceptor txi = new TransactionInterceptor();
        Mock fact = new Mock(TransactionFactory.class);
        Mock tx = new Mock(Transaction.class);
        tx.expectAndReturn("isOpen", false);
        tx.expect("begin");
        tx.expectAndReturn("isOpen", true);
        tx.expect("commit");
        fact.expectAndReturn("build", tx.proxy());
        txi.setTransactionFactory((TransactionFactory) fact.proxy());

        InterceptionBroker broker = new InterceptionBroker();
        broker.addInterceptor(txi, new NamedSignature("put"));

        HashMap map = new HashMap();
        HashMap wrap = (HashMap) broker.wrap(map);

        wrap.put("a", "b");

        fact.verify();
        tx.verify();
    }

    public void testRollbackOnException() throws Exception
    {
        TransactionInterceptor txi = new TransactionInterceptor();
        Mock fact = new Mock(TransactionFactory.class);
        Mock tx = new Mock(Transaction.class);
        tx.expectAndReturn("isOpen", false);
        tx.expect("begin");
        tx.expectAndReturn("isOpen", true);
        tx.expect("rollback");
        fact.expectAndReturn("build", tx.proxy());
        txi.setTransactionFactory((TransactionFactory) fact.proxy());

        InterceptionBroker broker = new InterceptionBroker();
        broker.addInterceptor(txi, new NamedSignature("put"));

        HashMap map = new HashMap()
        {
            public Object put(Object key, Object value)
            {
                throw new Error("Catch me!");
            }

        };
        HashMap wrap = (HashMap) broker.wrap(map);
        try {
            wrap.put("1", "2");
            fail("should have thrown an error");
        }
        catch (Throwable t) {
            assertTrue(true);
        }

        fact.verify();
        tx.verify();
    }

    public void testTransactionAware() throws Exception
    {
        Mock fact = new Mock(TransactionFactory.class);
        Mock tx = new Mock(Transaction.class);
        tx.expectAndReturn("isOpen", false);
        tx.expect("begin");
        tx.expectAndReturn("isOpen", true);
        tx.expect("commit");
        fact.expectAndReturn("build", tx.proxy());

        TransactionInterceptor txi = new TransactionInterceptor();
        txi.setTransactionFactory((TransactionFactory) fact.proxy());

        InterceptionBroker broker = new InterceptionBroker();
        broker.addInterceptor(txi, new NamedSignature("foo"));

        TransactionAware ta = new TransactionAware();
        TransactionAware wrap = (TransactionAware) broker.wrap(ta);

        wrap.foo();

        assertTrue(ta.pass);
        fact.verify();
        tx.verify();
    }

    private class TransactionAware
    {
        boolean pass = false;

        public void setTransaction(Transaction tx)
        {
            this.pass = true;
        }

        public void foo() { }
    }
}
