package org.skife.intercept.mock;

import junit.framework.TestCase;

import java.util.HashMap;
import java.util.Map;
import java.io.Serializable;

import com.mockobjects.dynamic.AnyConstraintMatcher;

public class TestMocker extends TestCase
{
    private Mocker mock;
    private HashMap base;

    public void setUp()
    {
        this.base = new HashMap();
        this.mock = new Mocker(base);
    }

    public void testPass()
    {
        HashMap base = new HashMap();
        Mocker mocker = new Mocker(base);
        mocker.expect("put", new AnyConstraintMatcher());

        HashMap wrap = (HashMap) mocker.getProxy();

        wrap.put("1", "2");
        mocker.verify();
    }

    public void testFail()
    {
        HashMap base = new HashMap();
        Mocker mocker = new Mocker(base);
        mocker.expect("put", new AnyConstraintMatcher());

        Map wrap = (Map) mocker.getProxy();

        try
        {
            mocker.verify();
            fail("Should have thrown an error");
        }
        catch (Error e)
        {
            assertTrue(true);
        }
    }

    public void testUnexpectedCall()
    {
        HashMap base = new HashMap();
        Mocker mocker = new Mocker(base);

        HashMap wrap = (HashMap) mocker.getProxy();

        try
        {
            wrap.get("a");
            fail("Should have thrown an error");
        }
        catch (Exception e)
        {
            assertTrue(true);
        }
    }

    public void testInterface()
    {
        Mocker mock = new Mocker(Map.class);
        mock.expect("put", new AnyConstraintMatcher());
        ((Map) mock.getProxy()).put("1", "2");
        mock.verify();
    }

    public void testInterfaces()
    {
        Mocker mock = new Mocker(new Class[]{Map.class, Serializable.class});
        mock.expect("put", new AnyConstraintMatcher());
        Object wrap = mock.getProxy();
        assertTrue(wrap instanceof Serializable);
        assertTrue(wrap instanceof Map);
        ((Map) wrap).put("1", "2");
        mock.verify();
    }

    public void testExpectAndReturn()
    {
        HashMap base = new HashMap();
        Mocker mock = new Mocker(base);
        Object ret = new Object();
        mock.expectAndReturn("get", new AnyConstraintMatcher(), ret);
        Map wrap = (Map) mock.getProxy();
        Object urn = wrap.get("a");
        assertEquals(ret, urn);
        mock.verify();
    }


    public void testExpectVoidPass()
    {
        mock.expect("toString");
        mock.getProxy().toString();
        mock.verify();
    }

    public void testExpectVoidFail()
    {
        mock.expect("toString");
        try
        {
            mock.verify();
            fail("Should have errored");
        }
        catch (Error e)
        {
            assertTrue(true);
        }
    }

    public void testExpectArgIntPass()
    {
        TestThing base = new TestThing();
        Mocker mock = new Mocker(base);
        mock.expectAndReturn("getInt", new AnyConstraintMatcher(), 1);
        assertEquals(1,((TestThing)mock.getProxy()).getInt("Hi"));
        mock.verify();
    }

    public void testExpectArgIntFail()
    {
        TestThing base = new TestThing();
        Mocker mock = new Mocker(base);
        mock.expectAndReturn("getInt", new AnyConstraintMatcher(), 1);
        //assertEquals(1,((TestThing)mock.getProxy()).getInt("Hi"));
        try
        {
            mock.verify();
            fail("Should have errored");
        }
        catch (Error e)
        {
            assertTrue(true);
        }
    }

    public void testExpectAndReturnVoidPass()
    {
        mock.expectAndReturn("toString", "hello");
        assertEquals("hello", mock.getProxy().toString());
        mock.verify();
    }

    public void testExpectAndReturnVoidFail()
    {
        mock.expectAndReturn("toString", "hello");
        try
        {
            mock.verify();
            fail("Should have errored");
        }
        catch (Error e)
        {
            assertTrue(true);
        }
    }

    public void testExpectAndReturnObjBoolean()
    {
        TestThing base = new TestThing();
        Mocker mock = new Mocker(base);
        mock.expectAndReturn("getBool", new AnyConstraintMatcher(), true);
        assertTrue(((TestThing)mock.getProxy()).getBool("foo"));
        mock.verify();
    }

    public class TestThing
    {
        public int getInt() { return 0; }
        public int getInt(Object foo) { return 0; }
        public boolean getBool() { return false; }
        public boolean getBool(Object foo) { return false; }
    }
}
