package org.skife.intercept.signatures;

import junit.framework.TestCase;
import org.skife.intercept.InterceptionBroker;
import org.skife.intercept.MockInterceptor;

import java.util.HashMap;
import java.util.Map;

public class TestClassSignature extends TestCase
{
    InterceptionBroker broker;

    public void setUp()
    {
        broker = new InterceptionBroker();
    }

    public void testMatchSame() throws Exception
    {
        HashMap base = new HashMap();
        MockInterceptor mock = new MockInterceptor();
        broker.addInterceptor(mock, new ClassSignature(HashMap.class));
        HashMap wrap = (HashMap) broker.wrap(base);
        wrap.put("1", "a");
        assertTrue(mock.value);
    }

    public void testMatchInterface() throws Exception
    {
        HashMap base = new HashMap();
        MockInterceptor mock = new MockInterceptor();
        broker.addInterceptor(mock, new ClassSignature(Map.class));
        HashMap wrap = (HashMap) broker.wrap(base);
        wrap.put("1", "a");
        assertTrue(mock.value);
    }

    public void testMatchSubclass() throws Exception
    {
        MyHashMap base = new MyHashMap();
        MockInterceptor mock = new MockInterceptor();
        broker.addInterceptor(mock, new ClassSignature(HashMap.class));
        HashMap wrap = (HashMap) broker.wrap(base);
        wrap.put("1", "a");
        assertTrue(mock.value);
    }

    public void testMatchOverride() throws Exception
    {
        MyHashMap base = new MyHashMap();
        MockInterceptor mock = new MockInterceptor();
        broker.addInterceptor(mock, new ClassSignature(HashMap.class));
        HashMap wrap = (HashMap) broker.wrap(base);
        String r = (String)wrap.get("1");

        assertEquals("hello", r);
        assertTrue(mock.value);
    }

    public void testDoNotMatchSuperClass()
    {
        HashMap base = new HashMap();
        MockInterceptor mock = new MockInterceptor();
        broker.addInterceptor(mock, new ClassSignature(MyHashMap.class));
        HashMap wrap = (HashMap) broker.wrap(base);
        wrap.put("1", "a");
        assertFalse(mock.value);
    }

    public static class MyHashMap extends HashMap
    {
        public Object get(Object key)
        {
            return "hello";
        }
    }
}
